@@ -6,7 +6,6 @@ bag-of-word counts.
 # bring model classes directly into package namespace, to save some typing
 from .hdpmodel import HdpModel
 from .ldamodel import LdaModel
-from .ldamallet import LdaMallet
 from .lsimodel import LsiModel
 from .tfidfmodel import TfidfModel
 from .rpmodel import RpModel
@@ -14,9 +13,10 @@ from .logentropy_model import LogEntropyModel
 from .word2vec import Word2Vec
 from .doc2vec import Doc2Vec
 from .ldamulticore import LdaMulticore
-from .dtmmodel import DtmModel
 from .phrases import Phrases
 
+from . import wrappers
+
 from gensim import interfaces, utils
 
 
