@@ -476,7 +476,6 @@ class Similarity(interfaces.SimilarityABC):
 #endclass Similarity
 
 
-
 class MatrixSimilarity(interfaces.SimilarityABC):
     """
     Compute similarity against a corpus of documents by storing the index matrix
@@ -491,7 +490,7 @@ class MatrixSimilarity(interfaces.SimilarityABC):
     See also `Similarity` and `SparseMatrixSimilarity` in this module.
 
     """
-    def __init__(self, corpus, num_best=None, dtype=numpy.float32, num_features=None, chunksize=256):
+    def __init__(self, corpus, num_best=None, dtype=numpy.float32, num_features=None, chunksize=256, corpus_len=None):
         """
         `num_features` is the number of features in the corpus (will be determined
         automatically by scanning the corpus if not specified). See `Similarity`
@@ -506,18 +505,19 @@ class MatrixSimilarity(interfaces.SimilarityABC):
         self.num_best = num_best
         self.normalize = True
         self.chunksize = chunksize
+        if corpus_len is None:
+            corpus_len = len(corpus)
 
         if corpus is not None:
             if self.num_features <= 0:
                 raise ValueError("cannot index a corpus with zero features (you must specify either `num_features` or a non-empty corpus in the constructor)")
-            logger.info("creating matrix for %s documents and %i features" %
-                         (len(corpus), num_features))
-            self.index = numpy.empty(shape=(len(corpus), num_features), dtype=dtype)
+            logger.info("creating matrix with %i documents and %i features", corpus_len, num_features)
+            self.index = numpy.empty(shape=(corpus_len, num_features), dtype=dtype)
             # iterate over corpus, populating the numpy index matrix with (normalized)
             # document vectors
             for docno, vector in enumerate(corpus):
                 if docno % 1000 == 0:
-                    logger.debug("PROGRESS: at document #%i/%i" % (docno, len(corpus)))
+                    logger.debug("PROGRESS: at document #%i/%i", docno, corpus_len)
                 # individual documents in fact may be in numpy.scipy.sparse format as well.
                 # it's not documented because other it's not fully supported throughout.
                 # the user better know what he's doing (no normalization, must
@@ -530,11 +530,9 @@ class MatrixSimilarity(interfaces.SimilarityABC):
                     vector = matutils.unitvec(matutils.sparse2full(vector, num_features))
                 self.index[docno] = vector
 
-
     def __len__(self):
         return self.index.shape[0]
 
-
     def get_similarities(self, query):
         """
         Return similarity of sparse vector `query` to all documents in the corpus,
@@ -549,11 +547,12 @@ class MatrixSimilarity(interfaces.SimilarityABC):
         """
         is_corpus, query = utils.is_corpus(query)
         if is_corpus:
-            query = numpy.asarray([matutils.sparse2full(vec, self.num_features) for vec in query],
-                                  dtype=self.index.dtype)
+            query = numpy.asarray(
+                [matutils.sparse2full(vec, self.num_features) for vec in query],
+                dtype=self.index.dtype)
         else:
             if scipy.sparse.issparse(query):
-                query = query.toarray() # convert sparse to dense
+                query = query.toarray()  # convert sparse to dense
             elif isinstance(query, numpy.ndarray):
                 pass
             else:
@@ -563,10 +562,12 @@ class MatrixSimilarity(interfaces.SimilarityABC):
 
         # do a little transposition dance to stop numpy from making a copy of
         # self.index internally in numpy.dot (very slow).
-        result = numpy.dot(self.index, query.T).T # return #queries x #index
-        return result # XXX: removed casting the result from array to list; does anyone care?
-#endclass MatrixSimilarity
+        result = numpy.dot(self.index, query.T).T  # return #queries x #index
+        return result  # XXX: removed casting the result from array to list; does anyone care?
 
+    def __str__(self):
+        return "%s<%i docs, %i features>" % (self.__class__.__name__, len(self), self.index.shape[1])
+#endclass MatrixSimilarity
 
 
 class SparseMatrixSimilarity(interfaces.SimilarityABC):
