@@ -72,7 +72,7 @@ class TestLsiModel(unittest.TestCase):
     def testCorpusTransform(self):
         """Test lsi[corpus] transformation."""
         model = lsimodel.LsiModel(self.corpus, num_topics=2)
-        got = numpy.vstack(matutils.sparse2full(doc, 2) for doc in model[corpus])
+        got = numpy.vstack(matutils.sparse2full(doc, 2) for doc in model[self.corpus])
         expected = numpy.array([
             [ 0.65946639,  0.14211544],
             [ 2.02454305, -0.42088759],
@@ -220,6 +220,7 @@ class TestRpModel(unittest.TestCase):
 class TestLdaModel(unittest.TestCase):
     def setUp(self):
         self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+        self.class_ = ldamodel.LdaModel
 
     def testTransform(self):
         passed = False
@@ -228,8 +229,8 @@ class TestLdaModel(unittest.TestCase):
         # better random initialization
         for i in range(5): # restart at most 5 times
             # create the transformation model
-            model = ldamodel.LdaModel(id2word=dictionary, num_topics=2, passes=100)
-            model.update(corpus)
+            model = self.class_(id2word=dictionary, num_topics=2, passes=100)
+            model.update(self.corpus)
 
             # transform one document
             doc = list(corpus)[0]
@@ -244,46 +245,78 @@ class TestLdaModel(unittest.TestCase):
                             (i, sorted(vec), sorted(expected)))
         self.assertTrue(passed)
 
-    def testTopicSeeding(self):
-        passed = False
-        for topic in range(2):
-            # try seeding it both ways round, check you get the same
-            # topics out but with which way round they are depending
-            # on the way round they're seeded
-            for i in range(5): # restart at most 5 times
+    def testTopTopics(self):
+        # create the transformation model
+        model = self.class_(id2word=dictionary, num_topics=2, passes=100)
+        model.update(self.corpus)
 
-                eta = numpy.ones((2, len(dictionary))) * 0.5
-                system = dictionary.token2id[u'system']
-                trees = dictionary.token2id[u'trees']
+        model.top_topics(self.corpus)
 
-                # aggressively seed the word 'system', in one of the
-                # two topics, 10 times higher than the other words
-                eta[topic, system] *= 10
+    def testPasses(self):
+        # long message includes the original error message with a custom one
+        self.longMessage = True
 
-                model = ldamodel.LdaModel(id2word=dictionary, num_topics=2, passes=200, eta=eta)
-                model.update(corpus)
+        # construct what we expect when passes aren't involved
+        test_rhots = list()
+        model = self.class_(id2word=dictionary, chunksize=1, num_topics=2)
+        final_rhot = lambda: pow(model.offset + (1 * model.num_updates) / model.chunksize, -model.decay)
 
-                topics = [dict((word, p) for p, word in model.show_topic(j)) for j in range(2)]
+        # generate 5 updates to test rhot on
+        for x in range(5):
+            model.update(self.corpus)
+            test_rhots.append(final_rhot())
 
-                # check that the word system in the topic we seeded, got a high weight,
-                # and the word 'trees' (the main word in the other topic) a low weight --
-                # and vice versa for the other topic (which we didn't seed with 'system')
-                result = [[topics[topic].get(u'system',0), topics[topic].get(u'trees',0)],
-                          [topics[1-topic].get(u'system',0), topics[1-topic].get(u'trees',0)]]
-                expected = [[0.385, 0.022],
-                            [0.025, 0.157]]
-                passed = numpy.allclose(result, expected, atol=1e-2)
-                if passed:
-                    break
-                logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" %
-                                (i, result, expected))
-            self.assertTrue(passed)
+        for passes in [1, 5, 10, 50, 100]:
+            model = self.class_(id2word=dictionary, chunksize=1, num_topics=2, passes=passes)
+            self.assertEqual(final_rhot(), 1.0)
+            # make sure the rhot matches the test after each update
+            for test_rhot in test_rhots:
+                model.update(self.corpus)
+
+                msg = ", ".join(map(str, [passes, model.num_updates, model.state.numdocs]))
+                self.assertAlmostEqual(final_rhot(), test_rhot, msg=msg)
+
+            self.assertEqual(model.state.numdocs, len(corpus) * len(test_rhots))
+            self.assertEqual(model.num_updates, len(corpus) * len(test_rhots))
+
+    # def testTopicSeeding(self):
+    #     for topic in range(2):
+    #         passed = False
+    #         for i in range(5):  # restart at most this many times, to mitigate LDA randomness
+    #             # try seeding it both ways round, check you get the same
+    #             # topics out but with which way round they are depending
+    #             # on the way round they're seeded
+    #             eta = numpy.ones((2, len(dictionary))) * 0.5
+    #             system = dictionary.token2id[u'system']
+    #             trees = dictionary.token2id[u'trees']
+
+    #             # aggressively seed the word 'system', in one of the
+    #             # two topics, 10 times higher than the other words
+    #             eta[topic, system] *= 10.0
+
+    #             model = self.class_(id2word=dictionary, num_topics=2, passes=200, eta=eta)
+    #             model.update(self.corpus)
+
+    #             topics = [dict((word, p) for p, word in model.show_topic(j, topn=None)) for j in range(2)]
+
+    #             # check that the word 'system' in the topic we seeded got a high weight,
+    #             # and the word 'trees' (the main word in the other topic) a low weight --
+    #             # and vice versa for the other topic (which we didn't seed with 'system')
+    #             passed = (
+    #                 (topics[topic][u'system'] > topics[topic][u'trees'])
+    #                 and
+    #                 (topics[1 - topic][u'system'] < topics[1 - topic][u'trees'])
+    #             )
+    #             if passed:
+    #                 break
+    #             logging.warning("LDA failed to converge on attempt %i (got %s)", i, topics)
+    #         self.assertTrue(passed)
 
     def testPersistence(self):
         fname = testfile()
-        model = ldamodel.LdaModel(self.corpus, num_topics=2)
+        model = self.class_(self.corpus, num_topics=2)
         model.save(fname)
-        model2 = ldamodel.LdaModel.load(fname)
+        model2 = self.class_.load(fname)
         self.assertEqual(model.num_topics, model2.num_topics)
         self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
         tstvec = []
@@ -291,9 +324,9 @@ class TestLdaModel(unittest.TestCase):
 
     def testPersistenceCompressed(self):
         fname = testfile() + '.gz'
-        model = ldamodel.LdaModel(self.corpus, num_topics=2)
+        model = self.class_(self.corpus, num_topics=2)
         model.save(fname)
-        model2 = ldamodel.LdaModel.load(fname, mmap=None)
+        model2 = self.class_.load(fname, mmap=None)
         self.assertEqual(model.num_topics, model2.num_topics)
         self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
         tstvec = []
@@ -301,13 +334,13 @@ class TestLdaModel(unittest.TestCase):
 
     def testLargeMmap(self):
         fname = testfile()
-        model = ldamodel.LdaModel(self.corpus, num_topics=2)
+        model = self.class_(self.corpus, num_topics=2)
 
         # simulate storing large arrays separately
         model.save(testfile(), sep_limit=0)
 
         # test loading the large model arrays with mmap
-        model2 = ldamodel.LdaModel.load(testfile(), mmap='r')
+        model2 = self.class_.load(testfile(), mmap='r')
         self.assertEqual(model.num_topics, model2.num_topics)
         self.assertTrue(isinstance(model2.expElogbeta, numpy.memmap))
         self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
@@ -316,121 +349,21 @@ class TestLdaModel(unittest.TestCase):
 
     def testLargeMmapCompressed(self):
         fname = testfile() + '.gz'
-        model = ldamodel.LdaModel(self.corpus, num_topics=2)
+        model = self.class_(self.corpus, num_topics=2)
 
         # simulate storing large arrays separately
         model.save(fname, sep_limit=0)
 
         # test loading the large model arrays with mmap
-        self.assertRaises(IOError, ldamodel.LdaModel.load, fname, mmap='r')
+        self.assertRaises(IOError, self.class_.load, fname, mmap='r')
 #endclass TestLdaModel
 
 
-class TestLdaMulticore(unittest.TestCase):
+class TestLdaMulticore(TestLdaModel):
     def setUp(self):
         self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
+        self.class_ = ldamulticore.LdaMulticore
 
-    def testTransform(self):
-        passed = False
-        # sometimes, LDA training gets stuck at a local minimum
-        # in that case try re-training the model from scratch, hoping for a
-        # better random initialization
-        for i in range(5): # restart at most 5 times
-            # create the transformation model
-            model = ldamulticore.LdaMulticore(id2word=dictionary, num_topics=2, passes=100)
-            model.update(corpus)
-
-            # transform one document
-            doc = list(corpus)[0]
-            transformed = model[doc]
-
-            vec = matutils.sparse2full(transformed, 2) # convert to dense vector, for easier equality tests
-            expected = [0.13, 0.87]
-            passed = numpy.allclose(sorted(vec), sorted(expected), atol=1e-2) # must contain the same values, up to re-ordering
-            if passed:
-                break
-            logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" %
-                            (i, sorted(vec), sorted(expected)))
-        self.assertTrue(passed)
-
-    def testTopicSeeding(self):
-        passed = False
-        for topic in range(2):
-            # try seeding it both ways round, check you get the same
-            # topics out but with which way round they are depending
-            # on the way round they're seeded
-            for i in range(5): # restart at most 5 times
-
-                eta = numpy.ones((2, len(dictionary))) * 0.5
-                system = dictionary.token2id[u'system']
-
-                # aggressively seed the word 'system', in one of the
-                # two topics, 10 times higher than the other words
-                eta[topic, system] *= 10
-
-                model = ldamulticore.LdaMulticore(id2word=dictionary, num_topics=2, passes=200, eta=eta)
-                model.update(corpus)
-
-                topics = [dict((word, p) for p, word in model.show_topic(j)) for j in range(2)]
-
-                # check that the word system in the topic we seeded, got a high weight,
-                # and the word 'trees' (the main word in the other topic) a low weight --
-                # and vice versa for the other topic (which we didn't seed with 'system')
-                result = [[topics[topic].get(u'system',0), topics[topic].get(u'trees',0)],
-                          [topics[1-topic].get(u'system',0), topics[1-topic].get(u'trees',0)]]
-                expected = [[0.385, 0.022],
-                            [0.025, 0.157]]
-                passed = numpy.allclose(result, expected, atol=1e-2)
-                if passed:
-                    break
-                logging.warning("LDA failed to converge on attempt %i (got %s, expected %s)" %
-                                (i, result, expected))
-            self.assertTrue(passed)
-
-    def testPersistence(self):
-        fname = testfile()
-        model = ldamulticore.LdaMulticore(self.corpus, num_topics=2)
-        model.save(fname)
-        model2 = ldamulticore.LdaMulticore.load(fname)
-        self.assertEqual(model.num_topics, model2.num_topics)
-        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
-        tstvec = []
-        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
-
-    def testPersistenceCompressed(self):
-        fname = testfile() + '.gz'
-        model = ldamulticore.LdaMulticore(self.corpus, num_topics=2)
-        model.save(fname)
-        model2 = ldamulticore.LdaMulticore.load(fname, mmap=None)
-        self.assertEqual(model.num_topics, model2.num_topics)
-        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
-        tstvec = []
-        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
-
-    def testLargeMmap(self):
-        fname = testfile()
-        model = ldamulticore.LdaMulticore(self.corpus, num_topics=2)
-
-        # simulate storing large arrays separately
-        model.save(testfile(), sep_limit=0)
-
-        # test loading the large model arrays with mmap
-        model2 = ldamulticore.LdaModel.load(testfile(), mmap='r')
-        self.assertEqual(model.num_topics, model2.num_topics)
-        self.assertTrue(isinstance(model2.expElogbeta, numpy.memmap))
-        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
-        tstvec = []
-        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
-
-    def testLargeMmapCompressed(self):
-        fname = testfile() + '.gz'
-        model = ldamulticore.LdaMulticore(self.corpus, num_topics=2)
-
-        # simulate storing large arrays separately
-        model.save(fname, sep_limit=0)
-
-        # test loading the large model arrays with mmap
-        self.assertRaises(IOError, ldamulticore.LdaModel.load, fname, mmap='r')
 #endclass TestLdaMulticore
 
 
@@ -608,7 +541,6 @@ class TestLogEntropyModel(unittest.TestCase):
 #endclass TestLogEntropyModel
 
 
-
 if __name__ == '__main__':
     logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.DEBUG)
     unittest.main()
