@@ -31,6 +31,7 @@ class LeeCorpus(object):
             for line in f:
                 yield utils.simple_preprocess(line)
 
+list_corpus = list(LeeCorpus())
 
 sentences = [
     ['human', 'interface', 'computer'],
@@ -173,6 +174,45 @@ class TestWord2VecModel(unittest.TestCase):
             self.assertFalse((unlocked1==model.syn0[1]).all())  # unlocked vector should vary
             self.assertTrue((locked0==model.syn0[0]).all())     # locked vector should not vary
 
+    def model_sanity(self, model, train=True):
+        """Even tiny models trained on LeeCorpus should pass these sanity checks"""
+        # run extra before/after training tests if train=True
+        if train:
+            model.build_vocab(list_corpus)
+            orig0 = numpy.copy(model.syn0[0])
+            model.train(list_corpus)
+            self.assertFalse((orig0==model.syn0[1]).all())  # vector should vary after training
+        sims = model.most_similar('war', topn=len(model.index2word))
+        t_rank = [word for word, score in sims].index('terrorism')
+        # in >200 calibration runs w/ calling parameters, 'terrorism' in 50-most_sim for 'war'
+        self.assertLess(t_rank, 50)
+        war_vec = model['war']
+        sims2 = model.most_similar([war_vec], topn=51)
+        self.assertTrue('war' in [word for word, score in sims2])
+        self.assertTrue('terrorism' in [word for word, score in sims2])
+
+    def test_sg_hs(self):
+        """Test skipgram w/ hierarchical softmax"""
+        model = word2vec.Word2Vec(sg=1, window=4, hs=1, negative=0, min_count=5, iter=10, workers=2)
+        self.model_sanity(model)
+
+    def test_sg_neg(self):
+        """Test skipgram w/ negative sampling"""
+        model = word2vec.Word2Vec(sg=1, window=4, hs=0, negative=15, min_count=5, iter=10, workers=2)
+        self.model_sanity(model)
+
+    def test_cbow_hs(self):
+        """Test CBOW w/ hierarchical softmax"""
+        model = word2vec.Word2Vec(sg=0, cbow_mean=1, alpha=0.05, window=5, hs=1, negative=0,
+                                  min_count=5, iter=10, workers=2)
+        self.model_sanity(model)
+
+    def test_cbow_neg(self):
+        """Test CBOW w/ negative sampling"""
+        model = word2vec.Word2Vec(sg=0, cbow_mean=1, alpha=0.05, window=5, hs=0, negative=15,
+                                  min_count=5, iter=10, workers=2)
+        self.model_sanity(model)
+
     def testTrainingCbow(self):
         """Test CBOW word2vec training."""
         # to test training, make the corpus larger by repeating its sentences over and over
@@ -316,6 +356,12 @@ class TestWord2VecSentenceIterators(unittest.TestCase):
                     self.assertEqual(words, utils.to_unicode(orig.readline()).split())
 #endclass TestWord2VecSentenceIterators
 
+if not hasattr(TestWord2VecModel, 'assertLess'):
+    # workaround for python 2.6
+    def assertLess(self, a, b, msg=None):
+        self.assertTrue(a < b, msg="%s is not less than %s" % (a, b))
+
+    setattr(TestWord2VecModel, 'assertLess', assertLess)
 
 if __name__ == '__main__':
     logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.DEBUG)
